@@ -15,7 +15,7 @@ from ._internal_init import *
 # ----------------------------------------------------------------------------
 
 if _ASDF_SETUP_ is False:
-    __all__ = ['AsdfFile', 'AsdfType', 'AsdfExtension',
+    __all__ = ['AsdfFile', 'CustomType', 'AsdfExtension',
                'Stream', 'open', 'test', 'commands',
                'ValidationError']
 
@@ -35,7 +35,7 @@ if _ASDF_SETUP_ is False:
         raise ImportError("asdf requires numpy")
 
     from .asdf import AsdfFile
-    from .asdftypes import AsdfType
+    from .asdftypes import CustomType
     from .extension import AsdfExtension
     from .stream import Stream
     from . import commands
