@@ -10,8 +10,6 @@ import re
 
 import numpy as np
 
-from .extern import semver
-
 from . import block
 from . import constants
 from . import extension
@@ -44,7 +42,9 @@ class AsdfFile(versioning.VersionedMixin):
     """
     The main class that represents a ASDF file.
     """
-    def __init__(self, tree=None, uri=None, extensions=None, version=None):
+    def __init__(self, tree=None, uri=None, extensions=None, version=None,
+        ignore_version_mismatch=True, ignore_unrecognized_tag=False,
+        copy_arrays=False):
         """
         Parameters
         ----------
@@ -67,7 +67,20 @@ class AsdfFile(versioning.VersionedMixin):
             The ASDF version to use when writing out.  If not
             provided, it will write out in the latest version
             supported by asdf.
+
+        ignore_version_mismatch : bool, optional
+            When `True`, do not raise warnings for mismatched schema versions.
+            Set to `True` by default.
+
+        ignore_unrecognized_tag : bool, optional
+            When `True`, do not raise warnings for unrecognized tags. Set to
+            `False` by default.
+
+        copy_arrays : bool, optional
+            When `False`, when reading files, attempt to memmap underlying data
+            arrays when possible.
         """
+
         if extensions is None or extensions == []:
             self._extensions = extension._builtin_extension_list
         else:
@@ -79,9 +92,14 @@ class AsdfFile(versioning.VersionedMixin):
                 extensions.insert(0, extension.BuiltinExtension())
                 self._extensions = extension.AsdfExtensionList(extensions)
 
+        self._ignore_version_mismatch = ignore_version_mismatch
+        self._ignore_unrecognized_tag = ignore_unrecognized_tag
+
+        self._file_format_version = None
+
         self._fd = None
         self._external_asdf_by_uri = {}
-        self._blocks = block.BlockManager(self)
+        self._blocks = block.BlockManager(self, copy_arrays=copy_arrays)
         self._uri = None
         if tree is None:
             self.tree = {}
@@ -123,7 +141,7 @@ class AsdfFile(versioning.VersionedMixin):
     @property
     def file_format_version(self):
         if self._file_format_version is None:
-            return semver.parse(self.versionspec['FILE_FORMAT'])
+            return versioning.AsdfVersion(self.version_map['FILE_FORMAT'])
         else:
             return self._file_format_version
 
@@ -248,6 +266,12 @@ class AsdfFile(versioning.VersionedMixin):
         self._validate(asdf_object)
         self._tree = asdf_object
 
+    def __getitem__(self, key):
+        return self._tree[key]
+
+    def __setitem__(self, key, value):
+        self._tree[key] = value
+
     @property
     def comments(self):
         """
@@ -346,16 +370,24 @@ class AsdfFile(versioning.VersionedMixin):
             the tree, only the most recent compression setting will be
             used, since all views share a single block.
 
-        array_compression : str or None
+        compression : str or None
             Must be one of:
 
+            - ``''`` or `None`: no compression
+
             - ``zlib``: Use zlib compression
 
             - ``bzp2``: Use bzip2 compression
 
+            - ``lz4``: Use lz4 compression
+
             - ``''`` or `None`: no compression
+
+            - ``input``: Use the same compression as in the file read.
+              If there is no prior file, acts as None.
+
         """
-        self.blocks[arr].compression = compression
+        self.blocks[arr].output_compression = compression
 
     def get_array_compression(self, arr):
         """
@@ -369,7 +401,7 @@ class AsdfFile(versioning.VersionedMixin):
         -------
         compression : str or None
         """
-        return self.blocks[arr].compression
+        return self.blocks[arr].output_compression
 
     @classmethod
     def _parse_header_line(cls, line):
@@ -381,7 +413,7 @@ class AsdfFile(versioning.VersionedMixin):
             raise ValueError("Does not appear to be a ASDF file.")
 
         try:
-            version = semver.parse(parts[1].decode('ascii'))
+            version = versioning.AsdfVersion(parts[1].decode('ascii'))
         except ValueError:
             raise ValueError(
                 "Unparseable version in ASDF file: {0}".format(parts[1]))
@@ -410,7 +442,7 @@ class AsdfFile(versioning.VersionedMixin):
             parts = comment.split()
             if len(parts) == 2 and parts[0] == constants.ASDF_STANDARD_COMMENT:
                 try:
-                    version = semver.parse(parts[1].decode('ascii'))
+                    version = versioning.AsdfVersion(parts[1].decode('ascii'))
                 except ValueError:
                     pass
                 else:
@@ -419,18 +451,17 @@ class AsdfFile(versioning.VersionedMixin):
         return None
 
     @classmethod
-    def _open_impl(cls, self, fd, uri=None, mode='r',
+    def _open_asdf(cls, self, fd, uri=None, mode='r',
                    validate_checksums=False,
                    do_not_fill_defaults=False,
-                   _get_yaml_content=False):
+                   _get_yaml_content=False,
+                   _force_raw_types=False):
+        """Attempt to populate AsdfFile data from file-like object"""
         fd = generic_io.get_file(fd, mode=mode, uri=uri)
-
         self._fd = fd
-
-        try:
-            header_line = fd.read_until(b'\r?\n', 2, "newline", include=True)
-        except ValueError:
-            raise ValueError("Does not appear to be a ASDF file.")
+        # The filename is currently only used for tracing warning information
+        self._fname = self._fd._uri if self._fd._uri else ''
+        header_line = fd.read_until(b'\r?\n', 2, "newline", include=True)
         self._file_format_version = cls._parse_header_line(header_line)
         self.version = self._file_format_version
 
@@ -444,7 +475,6 @@ class AsdfFile(versioning.VersionedMixin):
             self.version = version
 
         yaml_token = fd.read(4)
-        yaml_content = b''
         tree = {}
         has_blocks = False
         if yaml_token == b'%YAM':
@@ -452,24 +482,22 @@ class AsdfFile(versioning.VersionedMixin):
                 constants.YAML_END_MARKER_REGEX, 7, 'End of YAML marker',
                 include=True, initial_content=yaml_token)
 
+            # For testing: just return the raw YAML content
             if _get_yaml_content:
                 yaml_content = reader.read()
-            else:
-                # We parse the YAML content into basic data structures
-                # now, but we don't do anything special with it until
-                # after the blocks have been read
-                tree = yamlutil.load_tree(reader, self)
+                fd.close()
+                return yaml_content
+
+            # We parse the YAML content into basic data structures
+            # now, but we don't do anything special with it until
+            # after the blocks have been read
+            tree = yamlutil.load_tree(reader, self, self._ignore_version_mismatch)
             has_blocks = fd.seek_until(constants.BLOCK_MAGIC, 4, include=True)
         elif yaml_token == constants.BLOCK_MAGIC:
             has_blocks = True
         elif yaml_token != b'':
             raise IOError("ASDF file appears to contain garbage after header.")
 
-        # For testing: just return the raw YAML content
-        if _get_yaml_content:
-            fd.close()
-            return yaml_content
-
         if has_blocks:
             self._blocks.read_internal_blocks(
                 fd, past_magic=True, validate_checksums=validate_checksums)
@@ -479,18 +507,50 @@ class AsdfFile(versioning.VersionedMixin):
         if not do_not_fill_defaults:
             schema.fill_defaults(tree, self)
         self._validate(tree)
-        tree = yamlutil.tagged_tree_to_custom_tree(tree, self)
+        tree = yamlutil.tagged_tree_to_custom_tree(tree, self, _force_raw_types)
 
         self._tree = tree
         self.run_hook('post_read')
 
         return self
 
+    @classmethod
+    def _open_impl(cls, self, fd, uri=None, mode='r',
+                   validate_checksums=False,
+                   do_not_fill_defaults=False,
+                   _get_yaml_content=False,
+                   _force_raw_types=False):
+        """Attempt to open file-like object as either AsdfFile or AsdfInFits"""
+        if not is_asdf_file(fd):
+            try:
+                # TODO: this feels a bit circular, try to clean up. Also
+                # this introduces another dependency on astropy which may
+                # not be desireable.
+                from . import fits_embed
+                return fits_embed.AsdfInFits.open(fd, uri=uri,
+                            validate_checksums=validate_checksums,
+                            ignore_version_mismatch=self._ignore_version_mismatch,
+                            extensions=self._extensions)
+            except ValueError:
+                pass
+            raise ValueError(
+                "Input object does not appear to be ASDF file or FITS with " +
+                "ASDF extension")
+        return cls._open_asdf(self, fd, uri=uri, mode=mode,
+                validate_checksums=validate_checksums,
+                do_not_fill_defaults=do_not_fill_defaults,
+                _get_yaml_content=_get_yaml_content,
+                _force_raw_types=_force_raw_types)
+
     @classmethod
     def open(cls, fd, uri=None, mode='r',
              validate_checksums=False,
              extensions=None,
-             do_not_fill_defaults=False):
+             do_not_fill_defaults=False,
+             ignore_version_mismatch=True,
+             ignore_unrecognized_tag=False,
+             _force_raw_types=False,
+             copy_arrays=False):
         """
         Open an existing ASDF file.
 
@@ -520,17 +580,33 @@ class AsdfFile(versioning.VersionedMixin):
         do_not_fill_defaults : bool, optional
             When `True`, do not fill in missing default values.
 
+        ignore_version_mismatch : bool, optional
+            When `True`, do not raise warnings for mismatched schema versions.
+            Set to `True` by default.
+
+        ignore_unrecognized_tag : bool, optional
+            When `True`, do not raise warnings for unrecognized tags. Set to
+            `False` by default.
+
+        copy_arrays : bool, optional
+            When `False`, when reading files, attempt to memmap underlying data
+            arrays when possible.
+
         Returns
         -------
         asdffile : AsdfFile
             The new AsdfFile object.
         """
-        self = cls(extensions=extensions)
+        self = cls(extensions=extensions,
+                   ignore_version_mismatch=ignore_version_mismatch,
+                   ignore_unrecognized_tag=ignore_unrecognized_tag,
+                   copy_arrays=copy_arrays)
 
         return cls._open_impl(
             self, fd, uri=uri, mode=mode,
             validate_checksums=validate_checksums,
-            do_not_fill_defaults=do_not_fill_defaults)
+            do_not_fill_defaults=do_not_fill_defaults,
+            _force_raw_types=_force_raw_types)
 
     def _write_tree(self, tree, fd, pad_blocks):
         fd.write(constants.ASDF_MAGIC)
@@ -609,7 +685,7 @@ class AsdfFile(versioning.VersionedMixin):
         if hasattr(self, '_auto_inline'):
             del self._auto_inline
 
-    def update(self, all_array_storage=None, all_array_compression=None,
+    def update(self, all_array_storage=None, all_array_compression='input',
                auto_inline=None, pad_blocks=False, include_block_index=True,
                version=None):
         """
@@ -633,12 +709,17 @@ class AsdfFile(versioning.VersionedMixin):
             If provided, set the compression type on all binary blocks
             in the file.  Must be one of:
 
-            - ``''``: No compression.
+            - ``''`` or `None`: No compression.
 
             - ``zlib``: Use zlib compression.
 
             - ``bzp2``: Use bzip2 compression.
 
+            - ``lz4``: Use lz4 compression.
+
+            - ``input``: Use the same compression as in the file read.
+              If there is no prior file, acts as None
+
         auto_inline : int, optional
             When the number of elements in an array is less than this
             threshold, store the array as inline YAML, rather than a
@@ -734,7 +815,7 @@ class AsdfFile(versioning.VersionedMixin):
         finally:
             self._post_write(fd)
 
-    def write_to(self, fd, all_array_storage=None, all_array_compression=None,
+    def write_to(self, fd, all_array_storage=None, all_array_compression='input',
                  auto_inline=None, pad_blocks=False, include_block_index=True,
                  version=None):
         """
@@ -767,12 +848,17 @@ class AsdfFile(versioning.VersionedMixin):
             If provided, set the compression type on all binary blocks
             in the file.  Must be one of:
 
-            - ``''``: No compression.
+            - ``''`` or `None`: No compression.
 
             - ``zlib``: Use zlib compression.
 
             - ``bzp2``: Use bzip2 compression.
 
+            - ``lz4``: Use lz4 compression.
+
+            - ``input``: Use the same compression as in the file read.
+              If there is no prior file, acts as None.
+
         auto_inline : int, optional
             When the number of elements in an array is less than this
             threshold, store the array as inline YAML, rather than a
@@ -963,3 +1049,40 @@ class AsdfFile(versioning.VersionedMixin):
         except:
             self.tree['history'].pop()
             raise
+
+
+def is_asdf_file(fd):
+    """
+    Determine if fd is an ASDF file.
+
+    Reads the first five bytes and looks for the ``#ASDF`` string.
+
+    Parameters
+    ----------
+    fd : str, `~asdf.generic_io.GenericFile`
+
+    """
+    if isinstance(fd, generic_io.InputStream):
+        # If it's an InputStream let ASDF deal with it.
+        return True
+
+    to_close = False
+    if isinstance(fd, AsdfFile):
+        return True
+    elif isinstance(fd, generic_io.GenericFile):
+        pass
+    else:
+        try:
+            fd = generic_io.get_file(fd, mode='r', uri=None)
+            if not isinstance(fd, io.IOBase):
+                to_close = True
+        except ValueError:
+            return False
+    asdf_magic = fd.read(5)
+    if fd.seekable():
+        fd.seek(0)
+    if to_close:
+        fd.close()
+    if asdf_magic == constants.ASDF_MAGIC:
+        return True
+    return False
