@@ -4,18 +4,18 @@
 from __future__ import absolute_import, division, unicode_literals, print_function
 
 import bisect
-import imp
+import importlib
 import warnings
+import re
 
 import six
-
+from copy import copy
 
 from .compat import lru_cache
-from .extern import semver
 
 from . import tagged
 from . import util
-from . import versioning
+from .versioning import AsdfVersion, AsdfSpec, get_version_map
 
 
 __all__ = ['format_tag', 'AsdfTypeIndex', 'AsdfType']
@@ -25,11 +25,16 @@ _BASIC_PYTHON_TYPES = set(list(six.string_types) +
                           list(six.integer_types) +
                           [float, list, dict, tuple])
 
+# regex used to parse module name from optional version string
+MODULE_RE = re.compile(r'([a-zA-Z]+)(-(\d+\.\d+\.\d+))?')
+
 
 def format_tag(organization, standard, version, tag_name):
     """
     Format a YAML tag.
     """
+    if isinstance(version, AsdfSpec):
+        version = str(version.spec)
     return 'tag:{0}:{1}/{2}-{3}'.format(
         organization, standard, tag_name, version)
 
@@ -39,15 +44,15 @@ def split_tag_version(tag):
     Split a tag into its base and version.
     """
     name, version = tag.rsplit('-', 1)
-    version = semver.parse(version)
-    return name, (version['major'], version['minor'], version['patch'])
+    version = AsdfVersion(version)
+    return name, version
 
 
 def join_tag_version(name, version):
     """
     Join the root and version of a tag back together.
     """
-    return '{0}-{1}'.format(name, versioning.version_to_string(version))
+    return '{0}-{1}'.format(name, version)
 
 
 class _AsdfWriteTypeIndex(object):
@@ -76,6 +81,7 @@ class _AsdfWriteTypeIndex(object):
         self._version = version
 
         self._type_by_cls = {}
+        self._type_by_name = {}
         self._type_by_subclasses = {}
         self._types_with_dynamic_subclasses = {}
 
@@ -94,28 +100,27 @@ class _AsdfWriteTypeIndex(object):
             tag = join_tag_version(name, version)
             if tag in index._type_by_tag:
                 asdftype = index._type_by_tag[tag]
+                self._type_by_name[name] = asdftype
                 add_type(asdftype)
 
-        if version == 'latest':
+        if self._version == 'latest':
             for name, versions in six.iteritems(index._versions_by_type_name):
-                version = versions[-1]
-                add_by_tag(name, version)
+                add_by_tag(name, versions[-1])
         else:
             try:
-                version_map = versioning.get_version_map(version)
+                version_map = get_version_map(self._version)
             except ValueError:
                 raise ValueError(
                     "Don't know how to write out ASDF version {0}".format(
-                        version))
+                        self._version))
 
-            for name, version in six.iteritems(version_map['tags']):
-                add_by_tag(name, semver.parse(version))
+            for name, _version in six.iteritems(version_map['tags']):
+                add_by_tag(name, AsdfVersion(_version))
 
             # Now add any extension types that aren't known to the ASDF standard
             for name, versions in six.iteritems(index._versions_by_type_name):
-                if name not in version_map:
-                    version = versions[-1]
-                    add_by_tag(name, version)
+                if name not in self._type_by_name:
+                    add_by_tag(name, versions[-1])
 
         for asdftype in index._unnamed_types:
             add_type(asdftype)
@@ -156,9 +161,11 @@ class AsdfTypeIndex(object):
         self._type_by_tag = {}
         self._versions_by_type_name = {}
         self._best_matches = {}
+        self._real_tag = {}
         self._unnamed_types = set()
         self._hooks_by_type = {}
         self._all_types = set()
+        self._has_warned = {}
 
     def add_type(self, asdftype):
         """
@@ -209,7 +216,28 @@ class AsdfTypeIndex(object):
 
         return write_type_index.from_custom_type(custom_type)
 
-    def fix_yaml_tag(self, tag):
+    def _get_version_mismatch(self, name, version, latest_version):
+        warning_string = None
+
+        if (latest_version.major, latest_version.minor) != \
+                (version.major, version.minor):
+            warning_string = \
+                "'{}' with version {} found in file{{}}, but latest " \
+                "supported version is {}".format(
+                    name, version, latest_version)
+
+        return warning_string
+
+    def _warn_version_mismatch(self, ctx, tag, warning_string, fname):
+        if warning_string is not None:
+            # Ensure that only a single warning occurs per tag per AsdfFile
+            # TODO: If it is useful to only have a single warning per file on
+            # disk, then use `fname` in the key instead of `ctx`.
+            if not (ctx, tag) in self._has_warned:
+                warnings.warn(warning_string.format(fname))
+                self._has_warned[(ctx, tag)] = True
+
+    def fix_yaml_tag(self, ctx, tag, ignore_version_mismatch=True):
         """
         Given a YAML tag, adjust it to the best supported version.
 
@@ -218,57 +246,71 @@ class AsdfTypeIndex(object):
         the earliest understood version if none are less than the
         version in the file.
 
-        Raises a warning if it could not find a match where the major
-        and minor numbers are the same.
+        If ``ignore_version_mismatch==False``, this function raises a warning
+        if it could not find a match where the major and minor numbers are the
+        same.
         """
+        warning_string = None
+
+        name, version = split_tag_version(tag)
+
+        fname = " '{}'".format(ctx._fname) if ctx._fname else ''
+
         if tag in self._type_by_tag:
+            asdftype = self._type_by_tag[tag]
+            # Issue warnings for the case where there exists a class for the
+            # given tag due to the 'supported_versions' attribute being
+            # defined, but this tag is not the latest version of the type.
+            # This prevents 'supported_versions' from affecting the behavior of
+            # warnings that are purely related to YAML validation.
+            if not ignore_version_mismatch and hasattr(asdftype, '_latest_version'):
+                warning_string = self._get_version_mismatch(
+                    name, version, asdftype._latest_version)
+                self._warn_version_mismatch(ctx, tag, warning_string, fname)
             return tag
 
         if tag in self._best_matches:
             best_tag, warning_string = self._best_matches[tag]
 
-            if warning_string:
-                warnings.warn(warning_string)
+            if not ignore_version_mismatch:
+                self._warn_version_mismatch(ctx, tag, warning_string, fname)
 
             return best_tag
 
-        name, version = split_tag_version(tag)
         versions = self._versions_by_type_name.get(name)
         if versions is None:
             return tag
 
         # The versions list is kept sorted, so bisect can be used to
         # quickly find the best option.
-
         i = bisect.bisect_left(versions, version)
         i = max(0, i - 1)
 
+        if not ignore_version_mismatch:
+            warning_string = self._get_version_mismatch(
+                name, version, versions[-1])
+            self._warn_version_mismatch(ctx, tag, warning_string, fname)
+
         best_version = versions[i]
-        if best_version[:2] == version[:2]:
-            # Major and minor match, so only patch and devel differs
-            # -- no need for alarm
-            warning_string = None
-        else:
-            warning_string = (
-                "'{0}' with version {1} found in file, but asdf only "
-                "understands version {2}.".format(
-                    name,
-                    semver.format_version(*version),
-                    semver.format_version(*best_version)))
-
-        if warning_string:
-            warnings.warn(warning_string)
-
         best_tag = join_tag_version(name, best_version)
         self._best_matches[tag] = best_tag, warning_string
+        if tag != best_tag:
+            self._real_tag[best_tag] = tag
         return best_tag
 
-    def from_yaml_tag(self, tag):
+    def get_real_tag(self, tag):
+        if tag in self._real_tag:
+            return self._real_tag[tag]
+        elif tag in self._type_by_tag:
+            return tag
+        return None
+
+    def from_yaml_tag(self, ctx, tag):
         """
         From a given YAML tag string, return the corresponding
         AsdfType definition.
         """
-        tag = self.fix_yaml_tag(tag)
+        tag = self.fix_yaml_tag(ctx, tag)
         return self._type_by_tag.get(tag)
 
     @lru_cache(5)
@@ -308,34 +350,42 @@ _all_asdftypes = set()
 
 def _from_tree_tagged_missing_requirements(cls, tree, ctx):
     # A special version of AsdfType.from_tree_tagged for when the
-    # required dependencies for an AsdfType are missing.  Shows a
-    # warning, and then returns the raw dictionary.
-    plural = 's' if len(cls.requires) else ''
-    warnings.warn("{0} package{1} is required to instantiate '{2}'".format(
-        util.human_list(cls.requires), plural, tree._tag))
-    return tree
+    # required dependencies for an AsdfType are missing.
+    plural, verb = ('s', 'are') if len(cls.requires) else ('', 'is')
+    message = "{0} package{1} {2} required to instantiate '{3}'".format(
+        util.human_list(cls.requires), plural, verb, tree._tag)
+    # This error will be handled by yamlutil.tagged_tree_to_custom_tree, which
+    # will cause a warning to be issued indicating that the tree failed to be
+    # converted.
+    raise TypeError(message)
 
 
-class AsdfTypeMeta(type):
+class ExtensionTypeMeta(type):
     """
-    Keeps track of `AsdfType` subclasses that are created, and stores
-    them in `AsdfTypeIndex`.
+    Custom class constructor for extension types.
     """
     _import_cache = {}
 
     @classmethod
     def _has_required_modules(cls, requires):
-        for mod in requires:
-            if mod in cls._import_cache:
-                if not cls._import_cache[mod]:
+        for string in requires:
+            has_module = True
+            match = MODULE_RE.match(string)
+            modname, _, version = match.groups()
+            if modname in cls._import_cache:
+                if not cls._import_cache[modname]:
                     return False
             try:
-                imp.find_module(mod)
+                module = importlib.import_module(modname)
+                if version and hasattr(module, '__version__'):
+                    if module.__version__ < version:
+                        has_module = False
             except ImportError:
-                cls._import_cache[mod] = False
-                return False
-            else:
-                cls._import_cache[mod] = True
+                has_module = False
+            finally:
+                cls._import_cache[modname] = has_module
+                if not has_module:
+                    return False
         return True
 
     @classmethod
@@ -347,6 +397,10 @@ class AsdfTypeMeta(type):
                 return getattr(base, name)
         return default
 
+    @property
+    def versioned_siblings(mcls):
+        return getattr(mcls, '__versioned_siblings') or []
+
     def __new__(mcls, name, bases, attrs):
         requires = mcls._find_in_bases(attrs, bases, 'requires', [])
         if not mcls._has_required_modules(requires):
@@ -364,7 +418,11 @@ class AsdfTypeMeta(type):
                 new_types.append(typ)
             attrs['types'] = new_types
 
-        cls = super(AsdfTypeMeta, mcls).__new__(mcls, name, bases, attrs)
+        cls = super(ExtensionTypeMeta, mcls).__new__(mcls, name, bases, attrs)
+
+        if hasattr(cls, 'version'):
+            if not isinstance(cls.version, (AsdfVersion, AsdfSpec)):
+                cls.version = AsdfVersion(cls.version)
 
         if hasattr(cls, 'name'):
             if isinstance(cls.name, six.string_types):
@@ -375,14 +433,47 @@ class AsdfTypeMeta(type):
             elif cls.name is not None:
                 raise TypeError("name must be string or list")
 
+        if hasattr(cls, 'supported_versions'):
+            if not isinstance(cls.supported_versions, (list, set)):
+                cls.supported_versions = [cls.supported_versions]
+            supported_versions = set()
+            for version in cls.supported_versions:
+                if not isinstance(version, (AsdfVersion, AsdfSpec)):
+                    version = AsdfVersion(version)
+                # This should cause an exception for invalid input
+                supported_versions.add(version)
+            # We need to convert back to a list here so that the 'in' operator
+            # uses actual comparison instead of hash equality
+            cls.supported_versions = list(supported_versions)
+            siblings = list()
+            for version in cls.supported_versions:
+                if version != cls.version:
+                    new_attrs = copy(attrs)
+                    new_attrs['version'] = version
+                    new_attrs['supported_versions'] = set()
+                    new_attrs['_latest_version'] = cls.version
+                    siblings.append(
+                       ExtensionTypeMeta. __new__(mcls, name, bases, new_attrs))
+            setattr(cls, '__versioned_siblings', siblings)
+
+        return cls
+
+
+class AsdfTypeMeta(ExtensionTypeMeta):
+    """
+    Keeps track of `AsdfType` subclasses that are created, and stores them in
+    `AsdfTypeIndex`.
+    """
+    def __new__(mcls, name, bases, attrs):
+        cls = super(AsdfTypeMeta, mcls).__new__(mcls, name, bases, attrs)
+        # Classes using this metaclass get added to the list of built-in
+        # extensions
         _all_asdftypes.add(cls)
 
         return cls
 
 
-@six.add_metaclass(AsdfTypeMeta)
-@six.add_metaclass(util.InheritDocstrings)
-class AsdfType(object):
+class ExtensionType(object):
     """
     The base class of all custom types in the tree.
 
@@ -406,6 +497,11 @@ class AsdfType(object):
     version : 3-tuple of int
         The version of the standard the type is defined in.
 
+    supported_versions : set
+        If provided, indicates explicit compatibility with the given set of
+        versions. Other versions of the same schema that are not included in
+        this set will not be converted to custom types with this class.
+
     yaml_tag : str
         The YAML tag to use for the type.  If not provided, it will be
         automatically generated from name, organization, standard and
@@ -428,6 +524,7 @@ class AsdfType(object):
     organization = 'stsci.edu'
     standard = 'asdf'
     version = (1, 0, 0)
+    supported_versions = set()
     types = []
     handle_dynamic_subclasses = False
     validators = {}
@@ -439,7 +536,7 @@ class AsdfType(object):
         return format_tag(
             cls.organization,
             cls.standard,
-            versioning.version_to_string(cls.version),
+            cls.version,
             name)
 
     @classmethod
@@ -477,3 +574,37 @@ class AsdfType(object):
         with the tag directly.
         """
         return cls.from_tree(tree.data, ctx)
+
+    @classmethod
+    def incompatible_version(cls, version):
+        """
+        If this tag class explicitly identifies compatible versions then this
+        checks whether a given version is compatible or not. Otherwise, all
+        versions are assumed to be compatible.
+
+        Child classes can override this method to affect how version
+        compatiblity for this type is determined.
+        """
+        if cls.supported_versions:
+            if version not in cls.supported_versions:
+                return True
+        return False
+
+
+@six.add_metaclass(AsdfTypeMeta)
+@six.add_metaclass(util.InheritDocstrings)
+class AsdfType(ExtensionType):
+    """
+    Base class for all built-in ASDF types. Types that inherit this class will
+    be automatically added to the list of built-ins. This should *not* be used
+    for user-defined extensions.
+    """
+
+@six.add_metaclass(ExtensionTypeMeta)
+@six.add_metaclass(util.InheritDocstrings)
+class CustomType(ExtensionType):
+    """
+    Base class for all user-defined types. Unlike classes that inherit
+    AsdfType, classes that inherit this class will *not* automatically be added
+    to the list of built-ins. This should be used for user-defined extensions.
+    """
