@@ -33,7 +33,7 @@ class BlockManager(object):
     """
     Manages the `Block`s associated with a ASDF file.
     """
-    def __init__(self, asdffile):
+    def __init__(self, asdffile, copy_arrays=False):
         self._asdffile = weakref.ref(asdffile)
 
         self._internal_blocks = []
@@ -50,6 +50,7 @@ class BlockManager(object):
 
         self._data_to_block_mapping = {}
         self._validate_checksums = False
+        self._memmap = not copy_arrays
 
     def __len__(self):
         """
@@ -128,7 +129,7 @@ class BlockManager(object):
             block._array_storage = array_storage
             self.add(block)
             if array_storage == 'streamed':
-                block.compression = None
+                block.output_compression = None
 
     @property
     def blocks(self):
@@ -206,7 +207,7 @@ class BlockManager(object):
     def _read_next_internal_block(self, fd, past_magic=False):
         # This assumes the file pointer is at the beginning of the
         # block, (or beginning + 4 if past_magic is True)
-        block = Block().read(
+        block = Block(memmap=self._memmap).read(
             fd, past_magic=past_magic,
             validate_checksum=self._validate_checksums)
         if block is not None:
@@ -284,10 +285,12 @@ class BlockManager(object):
             should be after the tree.
         """
         for block in self.internal_blocks:
-            if block.is_compressed:
+            if block.output_compression:
                 block.offset = fd.tell()
                 block.write(fd)
             else:
+                if block.input_compression:
+                    block.update_size()
                 padding = util.calculate_padding(
                     block.size, pad_blocks, fd.block_size)
                 block.allocated = block._size + padding
@@ -378,8 +381,8 @@ class BlockManager(object):
                 allow_unicode=True, encoding='utf-8')
 
     _re_index_content = re.compile(
-        b'^' + constants.INDEX_HEADER + b'\r?\n%YAML.*\.\.\.\r?\n?$')
-    _re_index_misc = re.compile(b'^[\n\r\x20-\x7f]+$')
+        br'^' + constants.INDEX_HEADER + br'\r?\n%YAML.*\.\.\.\r?\n?$')
+    _re_index_misc = re.compile(br'^[\n\r\x20-\x7f]+$')
 
     def read_block_index(self, fd, ctx):
         """
@@ -491,7 +494,7 @@ class BlockManager(object):
         # make sure it makes sense.
         fd.seek(offsets[-1], generic_io.SEEK_SET)
         try:
-            block = Block().read(fd)
+            block = Block(memmap=self._memmap).read(fd)
         except (ValueError, IOError):
             return
 
@@ -502,7 +505,8 @@ class BlockManager(object):
         # It seems we're good to go, so instantiate the UnloadedBlock
         # objects
         for offset in offsets[1:-1]:
-            self._internal_blocks.append(UnloadedBlock(fd, offset))
+            self._internal_blocks.append(
+                UnloadedBlock(fd, offset, memmap=self._memmap))
 
         # We already read the last block in the file -- no need to read it again
         self._internal_blocks.append(block)
@@ -550,9 +554,8 @@ class BlockManager(object):
         if all_array_storage:
             self.set_array_storage(block, all_array_storage)
 
-        all_array_compression = getattr(ctx, '_all_array_compression', None)
-        if all_array_compression:
-            block.compression = all_array_compression
+        all_array_compression = getattr(ctx, '_all_array_compression', 'input')
+        block.output_compression = all_array_compression
 
         auto_inline = getattr(ctx, '_auto_inline', None)
         if auto_inline:
@@ -642,6 +645,10 @@ class BlockManager(object):
             block = asdffile.blocks._internal_blocks[0]
             self.set_array_storage(block, 'external')
 
+        # Handle the case of inline data
+        elif isinstance(source, list):
+            block = Block(data=np.array(source), array_storage='inline')
+
         else:
             raise TypeError("Unknown source '{0}'".format(source))
 
@@ -756,7 +763,7 @@ class Block(object):
         ('checksum', '16s')
     ])
 
-    def __init__(self, data=None, uri=None, array_storage='internal'):
+    def __init__(self, data=None, uri=None, array_storage='internal', memmap=True):
         if isinstance(data, np.ndarray) and not data.flags.c_contiguous:
             self._data = np.ascontiguousarray(data)
         else:
@@ -766,8 +773,10 @@ class Block(object):
 
         self._fd = None
         self._offset = None
-        self._compression = None
+        self._input_compression = None
+        self._output_compression = 'input'
         self._checksum = None
+        self._should_memmap = memmap
         self._memmapped = False
 
         self.update_size()
@@ -794,7 +803,7 @@ class Block(object):
     @allocated.setter
     def allocated(self, allocated):
         self._allocated = allocated
-        
+
     @property
     def header_size(self):
         return self._header.size + constants.BLOCK_HEADER_BOILERPLATE_SIZE
@@ -823,16 +832,29 @@ class Block(object):
         return self._array_storage
 
     @property
-    def compression(self):
-        return self._compression
+    def input_compression(self):
+        """
+        The compression codec used to read the block.
+        """
+        return self._input_compression
 
-    @compression.setter
-    def compression(self, compression):
-        self._compression = mcompression.validate(compression)
+    @input_compression.setter
+    def input_compression(self, compression):
+        self._input_compression = mcompression.validate(compression)
 
     @property
-    def is_compressed(self):
-        return self._compression is not None
+    def output_compression(self):
+        """
+        The compression codec used to write the block.
+        :return:
+        """
+        if self._output_compression == 'input':
+            return self._input_compression
+        return self._output_compression
+
+    @output_compression.setter
+    def output_compression(self, compression):
+        self._output_compression = mcompression.validate(compression)
 
     @property
     def checksum(self):
@@ -879,15 +901,15 @@ class Block(object):
         updating the file in-place, otherwise the work is redundant.
         """
         if self._data is not None:
-            if six.PY2:
+            if six.PY2: # pragma: no cover
                 self._data_size = len(self._data.data)
             else:
                 self._data_size = self._data.data.nbytes
-            if not self.is_compressed:
+            if not self.output_compression:
                 self._size = self._data_size
             else:
                 self._size = mcompression.get_compressed_size(
-                    self._data, self.compression)
+                    self._data, self.output_compression)
         else:
             self._data_size = self._size = 0
 
@@ -947,16 +969,16 @@ class Block(object):
 
         # This is used by the documentation system, but nowhere else.
         self._flags = header['flags']
-        self.compression = header['compression']
+        self.input_compression = header['compression']
         self._set_checksum(header['checksum'])
 
-        if (self.compression is None and
-            header['used_size'] != header['data_size']):
+        if (self.input_compression is None and
+                header['used_size'] != header['data_size']):
             raise ValueError(
                 "used_size and data_size must be equal when no compression is used.")
 
         if (header['flags'] & constants.BLOCK_FLAG_STREAMED and
-            self.compression is not None):
+                self.input_compression is not None):
             raise ValueError(
                 "Compression set on a streamed block.")
 
@@ -971,7 +993,7 @@ class Block(object):
                 fd.fast_forward(-1)
                 self._array_storage = 'streamed'
                 self._data_size = self._size = self._allocated = \
-                                  (fd.tell() - self.data_offset) + 1
+                    (fd.tell() - self.data_offset) + 1
             else:
                 fd.fast_forward(header['allocated_size'])
                 self._allocated = header['allocated_size']
@@ -988,8 +1010,7 @@ class Block(object):
                 self._data_size = header['data_size']
                 self._size = header['used_size']
                 self._allocated = header['allocated_size']
-                self._data = self._read_data(
-                    fd, self._size, self._data_size, self.compression)
+                self._data = self._read_data(fd, self._size, self._data_size)
                 fd.fast_forward(self._allocated - self._size)
             fd.close()
 
@@ -1000,12 +1021,12 @@ class Block(object):
 
         return self
 
-    def _read_data(self, fd, used_size, data_size, compression):
-        if not compression:
+    def _read_data(self, fd, used_size, data_size):
+        if not self.input_compression:
             return fd.read_into_array(used_size)
         else:
             return mcompression.decompress(
-                fd, used_size, data_size, compression)
+                fd, used_size, data_size, self.input_compression)
 
     def write(self, fd):
         """
@@ -1019,13 +1040,16 @@ class Block(object):
             flags |= constants.BLOCK_FLAG_STREAMED
         elif self._data is not None:
             self.update_checksum()
-            if not fd.seekable() and self.is_compressed:
-                buff = io.BytesIO()
-                mcompression.compress(buff, self._data, self.compression)
-                self.allocated = self._size = buff.tell()
             data_size = self._data.nbytes
+            if not fd.seekable() and self.output_compression:
+                buff = io.BytesIO()
+                mcompression.compress(buff, self._data,
+                                      self.output_compression)
+                self.allocated = self._size = buff.tell()
             allocated_size = self.allocated
             used_size = self._size
+        self.input_compression = self.output_compression
+        assert allocated_size >= used_size
 
         if self.checksum is not None:
             checksum = self.checksum
@@ -1037,13 +1061,13 @@ class Block(object):
         fd.write(self._header.pack(
             flags=flags,
             compression=mcompression.to_compression_header(
-                self.compression),
+                self.output_compression),
             allocated_size=allocated_size,
             used_size=used_size, data_size=data_size,
             checksum=checksum))
 
         if self._data is not None:
-            if self.is_compressed:
+            if self.output_compression:
                 if not fd.seekable():
                     fd.write(buff.getvalue())
                 else:
@@ -1052,7 +1076,8 @@ class Block(object):
                     # and write the resulting size in the block
                     # header.
                     start = fd.tell()
-                    mcompression.compress(fd, self._data, self.compression)
+                    mcompression.compress(
+                        fd, self._data, self.output_compression)
                     end = fd.tell()
                     self.allocated = self._size = end - start
                     fd.seek(self.offset + 6)
@@ -1062,6 +1087,7 @@ class Block(object):
                         used_size=self._size)
                     fd.seek(end)
             else:
+                assert used_size == data_size
                 fd.write_array(self._data)
 
     @property
@@ -1078,15 +1104,15 @@ class Block(object):
             # Be nice and reset the file position after we're done
             curpos = self._fd.tell()
             try:
-                if not self.is_compressed and self._fd.can_memmap():
+                memmap = self._fd.can_memmap() and not self.input_compression
+                if self._should_memmap and memmap:
                     self._data = self._fd.memmap_array(
                         self.data_offset, self._size)
                     self._memmapped = True
                 else:
                     self._fd.seek(self.data_offset)
                     self._data = self._read_data(
-                        self._fd, self._size, self._data_size,
-                        self.compression)
+                        self._fd, self._size, self._data_size)
             finally:
                 self._fd.seek(curpos)
 
@@ -1113,14 +1139,16 @@ class UnloadedBlock(object):
     full-fledged block whenever the underlying data or more detail is
     requested.
     """
-    def __init__(self, fd, offset):
+    def __init__(self, fd, offset, memmap=True):
         self._fd = fd
         self._offset = offset
         self._data = None
         self._uri = None
         self._array_storage = 'internal'
-        self._compression = None
+        self._input_compression = None
+        self._output_compression = 'input'
         self._checksum = None
+        self._should_memmap = memmap
         self._memmapped = False
 
     def __len__(self):
