@@ -6,3 +6,5 @@ from __future__ import absolute_import, division, unicode_literals, print_functi
 from .exploded import *
 from .to_yaml import *
 from .defragment import *
+from .diff import *
+from .tags import *
