@@ -60,7 +60,7 @@ def _check_bytes(fd, mode):
         if not isinstance(x, bytes):
             return False
     elif 'w' in mode:
-        if six.PY2:
+        if six.PY2: # pragma: no cover
             if isinstance(fd, file):
                 if 'b' not in fd.mode:
                     return False
@@ -867,7 +867,7 @@ class InputStream(GenericFile):
         try:
             # See if Numpy can handle this as a real file first...
             return np.fromfile(self._fd, np.uint8, size)
-        except IOError:
+        except (IOError, AttributeError):
             # Else, fall back to reading into memory and then
             # returning the Numpy array.
             data = self.read(size)
@@ -1196,7 +1196,7 @@ def get_file(init, mode='r', uri=None):
         raise TypeError(
             "io.StringIO objects are not supported.  Use io.BytesIO instead.")
 
-    elif six.PY2 and isinstance(init, file):
+    elif six.PY2 and isinstance(init, file): # pragma: no cover
         if init.mode[0] not in mode:
             raise ValueError(
                 "File is opened as '{0}', but '{1}' was requested".format(
@@ -1216,10 +1216,6 @@ def get_file(init, mode='r', uri=None):
             return RealFile(init, mode, uri=uri)
 
     elif isinstance(init, io.IOBase):
-        if sys.version_info[:2] == (2, 6):  # pragma: no cover
-            raise ValueError(
-                "io.open file objects are not supported on Python 2.6")
-
         if (('r' in mode and not init.readable()) or
             ('w' in mode and not init.writable())):
             raise ValueError(
