@@ -7,6 +7,8 @@ import datetime
 import json
 import os
 
+from collections import OrderedDict
+
 import six
 from six.moves.urllib import parse as urlparse
 
@@ -16,7 +18,6 @@ from jsonschema.exceptions import ValidationError
 import yaml
 
 from .compat import lru_cache
-from .compat.odict import OrderedDict
 from . import constants
 from . import generic_io
 from . import reference
@@ -55,7 +56,7 @@ PYTHON_TYPE_TO_YAML_TAG = {
 }
 
 
-if six.PY2:
+if six.PY2: # pragma: no cover
     PYTHON_TYPE_TO_YAML_TAG[long] = 'int'
 
 
@@ -274,7 +275,7 @@ OrderedLoader.add_constructor(
     construct_mapping)
 
 
-if six.PY2:
+if six.PY2: # pragma: no cover
     # Load strings in as Unicode on Python 2
     OrderedLoader.add_constructor('tag:yaml.org,2002:str',
                                   OrderedLoader.construct_scalar)
@@ -284,7 +285,8 @@ if six.PY2:
 def _load_schema(url):
     with generic_io.get_file(url) as fd:
         if isinstance(url, six.text_type) and url.endswith('json'):
-            result = json.load(fd, object_pairs_hook=OrderedDict)
+            json_data = fd.read().decode('utf-8')
+            result = json.loads(json_data, object_pairs_hook=OrderedDict)
         else:
             result = yaml.load(fd, Loader=OrderedLoader)
     return result, fd.uri
@@ -359,7 +361,7 @@ def load_schema(url, resolver=None, resolve_references=False):
             if json_id is None:
                 json_id = url
             if isinstance(node, dict) and '$ref' in node:
-                suburl = generic_io.resolve_uri(json_id, node['$ref'])
+                suburl = generic_io.resolve_uri(json_id, resolver(node['$ref']))
                 parts = urlparse.urlparse(suburl)
                 fragment = parts.fragment
                 if len(fragment):
@@ -429,7 +431,7 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
     return validator
 
 
-if six.PY2:
+if six.PY2: # pragma: no cover
     def validate_large_literals(instance):
         """
         Validate that the tree has no large numeric literals.
