@@ -33,7 +33,7 @@ import jsonschema
 
 import yaml
 
-from ....tests import helpers
+from ....tests import helpers, CustomTestType
 from .... import asdf
 from .... import util
 
@@ -43,10 +43,25 @@ from .. import ndarray
 TEST_DATA_PATH = os.path.join(os.path.dirname(__file__), 'data')
 
 
-class CustomExtension:
+# These custom types and the custom extension are here purely for the purpose
+# of testing NDArray objects and making sure that they can be validated as part
+# of a nested hierarchy, and not just top-level objects.
+class CustomNdim(CustomTestType):
+    name = 'ndim'
+    organization = 'nowhere.org'
+    standard = 'custom'
+    version = '1.0.0'
+
+class CustomDatatype(CustomTestType):
+    name = 'datatype'
+    organization = 'nowhere.org'
+    standard = 'custom'
+    version = '1.0.0'
+
+class CustomExtension(object):
     @property
     def types(self):
-        return []
+        return [CustomNdim, CustomDatatype]
 
     @property
     def tag_mapping(self):
@@ -196,6 +211,21 @@ def test_auto_inline_recursive(tmpdir):
         tree, tmpdir, check_asdf, None, {'auto_inline': 64})
 
 
+def test_copy_inline():
+    yaml = """
+x0: !core/ndarray-1.0.0
+  data: [-1.0, 1.0]
+    """
+
+    buff = helpers.yaml_to_asdf(yaml)
+
+    with asdf.AsdfFile.open(buff) as infile:
+        with asdf.AsdfFile() as f:
+            f.tree['a'] = infile.tree['x0']
+            f.tree['b'] = f.tree['a']
+            f.write_to(io.BytesIO())
+
+
 def test_table(tmpdir):
     table = np.array(
         [(0, 1, (2, 3)), (4, 5, (6, 7))],
