@@ -13,7 +13,7 @@ except ImportError:
 else:
     HAS_ASTROPY = True
     from astropy.utils import minversion
-    ASTROPY_13 = minversion(astropy, "1.3")
+    ASTROPY_13 = minversion(astropy, "1.3.dev16506")
     from astropy.modeling import models as astmodels
 
     test_models = [astmodels.Identity(2), astmodels.Polynomial1D(2, c0=1, c1=2, c2=3),
@@ -79,20 +79,6 @@ def test_name(tmpdir):
     helpers.assert_roundtrip_tree(tree, tmpdir, check)
 
 
-@pytest.mark.skipif('not HAS_ASTROPY')
-def test_domain(tmpdir):
-    def check(ff):
-        assert ff.tree['rot'].meta['domain'][0] == {
-            'lower': 0, 'upper': 1, 'includes_lower': True,
-            'includes_upper': False}
-
-    model = astmodels.Rotation2D(23)
-    model.meta['domain'] = [
-        {'lower': 0, 'upper': 1, 'includes_lower': True}]
-    tree = {'rot': model}
-    helpers.assert_roundtrip_tree(tree, tmpdir, check)
-
-
 @pytest.mark.skipif('not HAS_ASTROPY')
 def test_zenithal_with_arguments(tmpdir):
     tree = {
@@ -148,3 +134,11 @@ def test_tabular_model(tmpdir):
                                  fill_value=None, method='nearest')
     tree = {'model': model2}
     helpers.assert_roundtrip_tree(tree, tmpdir)
+
+
+@pytest.mark.skipif('not HAS_ASTROPY')
+def test_bounding_box(tmpdir):
+    model = astmodels.Shift(1) & astmodels.Shift(2)
+    model.bounding_box = ((1, 3), (2, 4))
+    tree = {'model': model}
+    helpers.assert_roundtrip_tree(tree, tmpdir)
