@@ -4,25 +4,17 @@
 from __future__ import absolute_import, division, unicode_literals, print_function
 
 import io
-
-try:
-    import astropy
-except ImportError:
-    HAS_ASTROPY = False
-else:
-    HAS_ASTROPY = True
-    from astropy import units as u
-
 import pytest
 
+astropy = pytest.importorskip('astropy')
+from astropy import units as u
+
 from .... import asdf
 from ....tests import helpers
 
 
 # TODO: Implement defunit
 
-
-@pytest.mark.skipif('not HAS_ASTROPY')
 def test_unit():
     yaml = """
 unit: !unit/unit-1.0.0 "2.1798721  10-18kg m2 s-2"
