@@ -16,16 +16,16 @@ class UnitType(AsdfType):
 
     @classmethod
     def to_tree(cls, node, ctx):
-        from astropy import units as u
+        from astropy.units import Unit, UnitBase
 
         if isinstance(node, six.string_types):
-            node = u.Unit(node, format='vounit', parse_strict='warn')
-        if isinstance(node, u.UnitBase):
+            node = Unit(node, format='vounit', parse_strict='warn')
+        if isinstance(node, UnitBase):
             return node.to_string(format='vounit')
         raise TypeError("'{0}' is not a valid unit".format(node))
 
     @classmethod
     def from_tree(cls, node, ctx):
-        from astropy import units as u
+        from astropy.units import Unit
 
-        return u.Unit(node, format='vounit', parse_strict='silent')
+        return Unit(node, format='vounit', parse_strict='silent')
