@@ -6,4 +6,5 @@ from __future__ import absolute_import, division, unicode_literals, print_functi
 
 def get_package_data():  # pragma: no cover
     return {
-        str(_PACKAGE_NAME_ + '.tests'): ['coveragerc', 'data/*.yaml']}
+        str(_PACKAGE_NAME_ + '.tests'):
+            ['coveragerc', 'data/*.yaml', 'data/*.json', 'data/*.fits']}
