@@ -54,6 +54,12 @@ def _roundtrip(tree, get_write_fd, get_read_fd,
                write_options={}, read_options={}):
     with get_write_fd() as fd:
         asdf.AsdfFile(tree).write_to(fd, **write_options)
+        # Work around the fact that generic_io's get_file doesn't have a way of
+        # determining whether or not the underlying file handle should be
+        # closed as part of the exit handler
+        if (six.PY3 and isinstance(fd._fd, io.FileIO)) or \
+                (six.PY2 and isinstance(fd._fd, file)):
+            fd._fd.close()
 
     with get_read_fd() as fd:
         ff = asdf.AsdfFile.open(fd, **read_options)
@@ -167,8 +173,6 @@ def test_open_fail4(tmpdir):
             generic_io.get_file(fd, mode='r')
 
 
-@pytest.mark.skipif(sys.version_info[:2] == (2, 6),
-                    reason="requires python 2.7 or later")
 def test_io_open(tree, tmpdir):
     path = os.path.join(str(tmpdir), 'test.asdf')
 
@@ -241,6 +245,7 @@ def test_streams2():
     assert len(x) == 60
 
 
+@helpers.remote_data
 @pytest.mark.skipif(sys.platform.startswith('win'),
                     reason="Windows firewall prevents test")
 def test_urlopen(tree, httpserver):
@@ -260,6 +265,7 @@ def test_urlopen(tree, httpserver):
         assert isinstance(next(ff.blocks.internal_blocks)._data, np.ndarray)
 
 
+@helpers.remote_data
 @pytest.mark.skipif(sys.platform.startswith('win'),
                     reason="Windows firewall prevents test")
 def test_http_connection(tree, httpserver):
@@ -284,6 +290,7 @@ def test_http_connection(tree, httpserver):
         ff.tree['science_data'][0] == 42
 
 
+@helpers.remote_data
 @pytest.mark.skipif(sys.platform.startswith('win'),
                     reason="Windows firewall prevents test")
 def test_http_connection_range(tree, rhttpserver):
@@ -300,11 +307,6 @@ def test_http_connection_range(tree, rhttpserver):
         return fd
 
     with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
-        if len(tree) == 4:
-            assert connection[0]._nreads == 0
-        else:
-            assert connection[0]._nreads == 6
-
         assert len(list(ff.blocks.internal_blocks)) == 2
         assert isinstance(next(ff.blocks.internal_blocks)._data, np.core.memmap)
         assert isinstance(next(ff.blocks.internal_blocks)._data, np.ndarray)
@@ -348,6 +350,7 @@ def test_exploded_filesystem_fail(tree, tmpdir):
                 helpers.assert_tree_match(tree, ff.tree)
 
 
+@helpers.remote_data
 @pytest.mark.skipif(sys.platform.startswith('win'),
                     reason="Windows firewall prevents test")
 def test_exploded_http(tree, httpserver):
@@ -778,3 +781,19 @@ def test_truncated_reader():
     assert tr.read(99) == (init + content[:95])
     assert tr.read(50) == content[95:105]
     assert tr.read() == b''
+
+
+def test_is_asdf(tmpdir):
+    # test fits
+    astropy = pytest.importorskip('astropy')
+    from astropy.io import fits
+
+    hdul = fits.HDUList()
+    phdu= fits.PrimaryHDU()
+    imhdu= fits.ImageHDU(data=np.arange(24).reshape((4,6)))
+    hdul.append(phdu)
+    hdul.append(imhdu)
+    path = os.path.join(str(tmpdir), 'test.fits')
+    hdul.writeto(path)
+    assert not asdf.is_asdf_file(path)
+    assert asdf.is_asdf_file(asdf.AsdfFile())
