@@ -8,6 +8,7 @@ import os
 
 import numpy as np
 from numpy.testing import assert_array_equal
+from astropy.modeling import models
 
 import pytest
 
@@ -18,6 +19,8 @@ from .. import block
 from .. import constants
 from .. import generic_io
 from .. import treeutil
+from .. import versioning
+from ..tests.helpers import assert_tree_match
 
 
 def _get_small_tree():
@@ -774,6 +777,21 @@ def test_atomic_write(tmpdir):
         ff.write_to(tmpfile)
 
 
+def test_overwrite(tmpdir):
+    # This is intended to reproduce the following issue:
+    # https://github.com/spacetelescope/asdf/issues/100
+    tmpfile = os.path.join(str(tmpdir), 'test.asdf')
+    aff = models.AffineTransformation2D(matrix=[[1, 2], [3, 4]])
+    f = asdf.AsdfFile()
+    f.tree['model'] = aff
+    f.write_to(tmpfile)
+    model = f.tree['model']
+
+    ff = asdf.AsdfFile()
+    ff.tree['model'] = model
+    ff.write_to(tmpfile)
+
+
 def test_walk_and_modify_remove_keys():
     tree = {
         'foo': 42,
@@ -1073,6 +1091,34 @@ def test_dots_but_no_block_index():
         assert len(ff.blocks) == 1
 
 
+def test_open_no_memmap(tmpdir):
+    tmpfile = os.path.join(str(tmpdir), 'random.asdf')
+
+    tree = {
+        'array': np.random.random((20, 20))
+    }
+
+    ff = asdf.AsdfFile(tree)
+    ff.write_to(tmpfile)
+
+    # Test that by default we use memmapped arrays when possible
+    with asdf.AsdfFile.open(tmpfile) as af:
+        array = af.tree['array']
+        # Make sure to access the block so that it gets loaded
+        x = array[0]
+        assert array.block._memmapped == True
+        assert isinstance(array.block._data, np.memmap)
+
+    # Test that if we ask for copy, we do not get memmapped arrays
+    with asdf.AsdfFile.open(tmpfile, copy_arrays=True) as af:
+        array = af.tree['array']
+        x = array[0]
+        assert array.block._memmapped == False
+        # We can't just check for isinstance(..., np.array) since this will
+        # be true for np.memmap as well
+        assert not isinstance(array.block._data, np.memmap)
+
+
 def test_invalid_version(tmpdir):
     content = b"""#ASDF 0.1.0
 %YAML 1.1
@@ -1084,3 +1130,56 @@ foo : bar
     with pytest.raises(ValueError):
         with asdf.AsdfFile.open(buff) as ff:
             pass
+
+
+def test_valid_version(tmpdir):
+    content = b"""#ASDF 1.0.0
+%YAML 1.1
+%TAG ! tag:stsci.edu:asdf/
+--- !core/asdf-1.0.0
+foo : bar
+..."""
+    buff = io.BytesIO(content)
+    with asdf.AsdfFile.open(buff) as ff:
+        version = ff.file_format_version
+
+    assert version.major == 1
+    assert version.minor == 0
+    assert version.patch == 0
+
+
+def test_default_version():
+    # See https://github.com/spacetelescope/asdf/issues/364
+
+    version_map = versioning.get_version_map(versioning.default_version)
+
+    ff = asdf.AsdfFile()
+    assert ff.file_format_version == version_map['FILE_FORMAT']
+
+
+def test_fd_not_seekable():
+    data = np.ones(1024)
+    b = block.Block(data=data)
+    fd = io.BytesIO()
+    fd.seekable = lambda: False
+    fd.write_array = lambda arr: fd.write(arr.tobytes())
+    fd.read_blocks = lambda us: [fd.read(us)]
+    fd.fast_forward = lambda offset: fd.seek(offset, 1)
+    b.output_compression = 'zlib'
+    b.write(fd)
+    fd.seek(0)
+    b = block.Block()
+    b.read(fd)
+    # We lost the information about the underlying array type,
+    # but still can compare the bytes.
+    assert b.data.tobytes() == data.tobytes()
+
+
+def test_top_level_tree():
+    tree = {'tree': _get_small_tree()}
+    ff = asdf.AsdfFile(tree)
+    assert_tree_match(ff.tree['tree'], ff['tree'])
+
+    ff2 = asdf.AsdfFile()
+    ff2['tree'] = _get_small_tree()
+    assert_tree_match(ff2.tree['tree'], ff2['tree'])
