@@ -6,14 +6,17 @@ from __future__ import absolute_import, division, unicode_literals, print_functi
 import numpy as np
 
 import six
-
 import yaml
+import warnings
+
+from collections import OrderedDict
 
-from . compat.odict import OrderedDict
 from . constants import YAML_TAG_PREFIX
 from . import schema
 from . import tagged
 from . import treeutil
+from . import asdftypes
+from . import versioning
 from . import util
 
 
@@ -143,12 +146,14 @@ class AsdfLoader(_yaml_base_loader):
     A specialized YAML loader that can construct "tagged basic Python
     data types" as implemented in the `tagged` module.
     """
+    ignore_version_mismatch = False
     def construct_object(self, node, deep=False):
         tag = node.tag
         if node.tag in self.yaml_constructors:
             return super(AsdfLoader, self).construct_object(node, deep=False)
         data = _yaml_to_base_type(node, self)
-        tag = self.ctx.type_index.fix_yaml_tag(tag)
+        tag = self.ctx.type_index.fix_yaml_tag(
+            self.ctx, tag, self.ignore_version_mismatch)
         data = tagged.tag_object(tag, data)
         return data
 
@@ -212,7 +217,7 @@ for scalar_type in util.iter_subclasses(np.integer):
 # ----------------------------------------------------------------------
 # Unicode fix on Python 2
 
-if six.PY2:
+if six.PY2: # pragma: no cover
     # This dumps Python unicode strings as regular YAML strings rather
     # than !!python/unicode. See http://pyyaml.org/ticket/11
     def _unicode_representer(dumper, value):
@@ -238,23 +243,56 @@ def custom_tree_to_tagged_tree(tree, ctx):
     return treeutil.walk_and_modify(tree, walker)
 
 
-def tagged_tree_to_custom_tree(tree, ctx):
+def tagged_tree_to_custom_tree(tree, ctx, force_raw_types=False):
     """
     Convert a tree containing only basic data types, annotated with
     tags, to a tree containing custom data types.
     """
+
     def walker(node):
+        if force_raw_types:
+            return node
+
         tag_name = getattr(node, '_tag', None)
-        if tag_name is not None:
-            tag_type = ctx.type_index.from_yaml_tag(tag_name)
-            if tag_type is not None:
-                return tag_type.from_tree_tagged(node, ctx)
+        if tag_name is None:
+            return node
+
+        tag_type = ctx.type_index.from_yaml_tag(ctx, tag_name)
+        # This means the tag did not correspond to any type in our type index.
+        if tag_type is None:
+            if not ctx._ignore_unrecognized_tag:
+                warnings.warn("{} is not recognized, converting to raw Python "
+                    "data structure".format(tag_name))
+            return node
+
+        real_tag = ctx.type_index.get_real_tag(tag_name)
+        real_tag_name, real_tag_version = asdftypes.split_tag_version(real_tag)
+        # This means that there is an explicit description of versions that are
+        # compatible with the associated tag class implementation, but the
+        # version we found does not fit that description.
+        if tag_type.incompatible_version(real_tag_version):
+            warnings.warn("Version {} of {} is not compatible with any "
+                "existing tag implementations".format(
+                    real_tag_version, real_tag_name))
+            return node
+
+        # If a tag class does not explicitly list compatible versions, then all
+        # versions of the corresponding schema are assumed to be compatible.
+        # Therefore we need to check to make sure whether the conversion is
+        # actually successful, and just return a raw Python data type if it is
+        # not.
+        try:
+            return tag_type.from_tree_tagged(node, ctx)
+        except TypeError as err:
+            warnings.warn("Failed to convert {} to custom type (detail: {}). "
+                "Using raw Python data structure instead".format(real_tag, err))
+
         return node
 
     return treeutil.walk_and_modify(tree, walker)
 
 
-def load_tree(stream, ctx):
+def load_tree(stream, ctx, ignore_version_mismatch=False):
     """
     Load YAML, returning a tree of objects.
 
@@ -266,6 +304,7 @@ def load_tree(stream, ctx):
     class AsdfLoaderTmp(AsdfLoader):
         pass
     AsdfLoaderTmp.ctx = ctx
+    AsdfLoaderTmp.ignore_version_mismatch = ignore_version_mismatch
 
     return yaml.load(stream, Loader=AsdfLoaderTmp)
 
