@@ -20,7 +20,7 @@ from asdf import AsdfFile
 from asdf.constants import ASDF_MAGIC, BLOCK_FLAG_STREAMED
 from asdf import versioning
 
-version_string = versioning.version_to_string(versioning.default_version)
+version_string = str(versioning.default_version)
 
 
 TMPDIR = tempfile.mkdtemp()
@@ -109,8 +109,8 @@ class AsdfDirective(Directive):
                             human_flags.append(val)
                     if len(human_flags):
                         lines.append('    flags: {0}'.format(' | '.join(human_flags)))
-                    if block.compression:
-                        lines.append('    compression: {0}'.format(block.compression))
+                    if block.input_compression:
+                        lines.append('    compression: {0}'.format(block.input_compression))
                     lines.append('    allocated_size: {0}'.format(allocated))
                     lines.append('    used_size: {0}'.format(size))
                     lines.append('    data_size: {0}'.format(data_size))
@@ -128,7 +128,7 @@ class AsdfDirective(Directive):
                     internal_blocks[-1].array_storage != 'streamed'):
                     buff = io.BytesIO()
                     ff.blocks.write_block_index(buff, ff)
-                    block_index = buff.getvalue()
+                    block_index = buff.getvalue().decode('utf-8')
                     literal = nodes.literal_block(block_index, block_index)
                     literal['language'] = 'yaml'
                     set_source_info(self, literal)
